# it is assumed that $steal is already defined (0 or 1)
# but let's provide a default anyway..
if ![info exists steal] {set steal 0}

#######################
### initializations ###
#######################

# withdraw . until we've built everything
wm withdraw .

# make it resizable, with decent minimum sizes
wm minsize . 300 100
wm geometry . +90+30
wm iconbitmap . @$hdrug_library/bitmaps/hdrug.xbm

set app_name Hdrug

set parser_exists 0
set generator_exists 0
set type_exists 0
set types {}
set preds {}
set sents {}
set objs 0
set lfs {}
set fs_started 0

# get info from Prolog (possibly overwriting previous variables)
prolog hdrug_startup_hook_begin

loadAppDefaults "$app_name" 100
wm title . "$app_name"
wm iconname . "$app_name"

########################
### The main widgets ###
########################

# MenuBar
frame .menu
pack .menu -side top -fill x

# ObjectBar
frame .bb -bd 2 -relief raised 
## -height 25
pack .bb -fill x

# Canvas
frame .cv
pack .cv -fill both -expand 1

# Sicstus
# large stdin/stdout shell (optional)
# Embedded xterm shell for ordinary prolog input/output
# updates are necc. otherwise error that parent window not is
# specified..
# the widthxheight depends on the size of xterm that you use
if $steal then {
    frame .shellbar
    frame .shell -width 840 -height 250
    pack .shellbar -fill x
    pack .shell
    # i don;t want to see the window at this moment, but I have
    # to make it visible for tksteal. For some windowmanagers
    # the following is a hack that makes it visible for tksteal
    # but invisible for the person sitting behind the terminal
    wm geometry . +10000+10000
    wm deiconify .
    update
    # steal it!
    xaccess eventreparent -window $env(WINDOWID) -parentwidget .shell
    # using tksteal command does not work

    # allow resizing of stolen window:
    button .shellbar.smaller -relief raised \
                -bitmap @$hdrug_library/bitmaps/SP_left.bm -command {
    .shell configure -width \
            [expr [lindex [.shell configure -width] 4]-50]
    }
    button .shellbar.wider -relief raised \
                -bitmap @$hdrug_library/bitmaps/SP_right.bm -command {
     .shell configure -width \
                [expr [lindex [.shell configure -width] 4]+50]
    }
    button .shellbar.higher -relief raised \
                -bitmap @$hdrug_library/bitmaps/SP_down.bm -command {
    .shell configure -height \
                [expr [lindex [.shell configure -height] 4]+50]
    }
    button .shellbar.lower -relief raised \
                -bitmap @$hdrug_library/bitmaps/SP_up.bm -command {
    .shell configure -height \
                [expr [lindex [.shell configure -height] 4]-50]
    }
    pack .shellbar.smaller .shellbar.lower .shellbar.higher \
                .shellbar.wider -side left
    # reset things
    wm withdraw .
    wm geometry . +90+30
    } {puts "OK, not stealing the XTERM containing the Sicstus prompt"}

# ButtonBar
# fill in whatever you like..
frame .t -relief raised
pack .t -side bottom -expand 0 -fill x

# end of slaves of .

############
## Canvas ##
############

canvas .cv.canvas -bd 2 -relief raised \
   -yscrollcommand ".cv.sy set" -xscrollcommand ".cv.sx set"\
   -scrollregion {-1c -1c 200c 100c}

# place a bitmap on the canvas
.cv.canvas create bitmap 20 20 -bitmap @$hdrug_library/bitmaps/hdrug.xbm

## the large canvas has scrollbars
scrollbar .cv.sx -relief sunken -orient horiz -command ".cv.canvas xview"
scrollbar .cv.sy -relief sunken -command ".cv.canvas yview"

pack .cv.sx -side bottom -fill x
pack .cv.sy -side right -fill y
pack .cv.canvas -fill both -expand 1

## the large canvas can also be `scrolled' by dragging 2nd mouse button
bind .cv.canvas <2> ".cv.canvas scan mark %x %y"
bind .cv.canvas <B2-Motion> ".cv.canvas scan dragto %x %y"
## the large canvas is cleared with third mouse button
bind .cv.canvas <3> ".cv.canvas delete all"

#############
## MenuBar ##
#############

menubutton .menu.file -text "File" -underline 0 -menu .menu.file.m
menu .menu.file.m

.menu.file.m add command -label "Compile" -underline 0 \
   -command {
            global fsBox
            set fsBox(pattern) *.pl
            FSBox {} {} {
              prolog "compile $fsBox(path)/$fsBox(name)"
              set fsBox(pattern) * 
            }
}

.menu.file.m add command -label "Reconsult" -underline 0 \
   -command {
            global fsBox
            set fsBox(pattern) *.pl
            FSBox {} {} {
              prolog "reconsult $fsBox(path)/$fsBox(name)"
	      set fsBox(pattern) * 
            }
}

.menu.file.m add command -label "Source TK/Tcl file" -underline 0 \
   -command {
            global fsBox
            set fsBox(pattern) *.tcl
            FSBox {} {} {
              source $fsBox(path)/$fsBox(name)
              set fsBox(pattern) *
            }
}

if !$steal then {
    .menu.file.m add command -label "Restart X" -underline 8 \
           -command {prolog restart_x}
}

if !$steal then {
    .menu.file.m add command -label "Restart X and steal the shell" \
           -underline 3 -command {prolog "restart_x 1"}
}

if !$steal then {
    .menu.file.m add command -label "Quit X (keep Prolog alive)" \
           -underline 0 -command {prolog halt_x}
}

.menu.file.m add command -label "Halt (both X and Prolog)" -underline 0 \
	-command exit_prolog
pack .menu.file -side left

menubutton .menu.debug -text "Debug" -underline 0 -menu .menu.debug.m
menu .menu.debug.m
.menu.debug.m add command -label "Nodebug" -underline 0 \
  -command {prolog nodebug}
.menu.debug.m add command -label "Debug" \
  -command {prolog debug} -underline 0
.menu.debug.m add command -label "Remove spypoints" \
  -command {prolog nospyall} -underline 0
.menu.debug.m add command -label "Spy predicate" -underline 0\
  -command {prolog "send_spy spy \
              [SP_question .g question {Predicate to spy:} {}]"}
.menu.debug.m add command -label "Unspy predicate" -underline 0 \
  -command {prolog "send_spy nospy \
              [SP_question .g question {Predicate to unspy:} {}]"}
.menu.debug.m add command -label "statistics" \
  -command {prolog statistics}
pack .menu.debug -side left

menubutton .menu.help -text "Help" -underline 0 -menu .menu.help.m
menu .menu.help.m
.menu.help.m add command -label "Help" -underline 0 \
   -command {prolog help}
.menu.help.m add command -label "About" -underline 0 \
   -command {
    global dir
    tk_dialog .dialog "About" \
        {All flames to vannoord@let.rug.nl (c) RUG 1994} \
        @$hdrug_library/bitmaps/SP_vannoord.bm 0 {Seen enough?}
}

.menu.help.m add command -label "Version" -underline 0 \
   -command {prolog version}
pack .menu.help -side right

menubutton .menu.options -text "Options" -underline 0 -menu .menu.options.m
menu .menu.options.m
.menu.options.m add cascade -label "Top Category" \
    -menu .menu.options.top
if $parser_exists then {
    .menu.options.m add cascade -label "Parser" \
        -menu .menu.options.parser 
}
if $generator_exists then {
    .menu.options.m add cascade -label "Generator" \
    -menu .menu.options.generator 
}

menu .menu.options.top -postcommand {
    prolog send_topcats
    catch {.menu.options.top delete 0 last}
    foreach i $topcats {
        .menu.options.top add command -label $i \
               -command "SP_receive_flag top_features $i"
    }
    .menu.options.top add command -label "<No Top>" \
               -command "SP_receive_flag top_features undefined"
}

menu .menu.options.parser -postcommand {
    prolog {send_list parser_mode parsers}
    catch {.menu.options.parser delete 0 last}
    foreach i $parsers {
        .menu.options.parser add command -label $i \
               -command "SP_receive_flag parser $i"
    }
}
menu .menu.options.generator -postcommand {
    prolog {send_list generator_mode generators}
    catch {.menu.options.generator delete 0 last}
    foreach i $generators {
        .menu.options.generator add command -label $i \
               -command "SP_receive_flag generator $i"
    }
}

pack .menu.options -side left

menubutton .menu.parse -text "Parse" -underline 0 -menu .menu.parse.m
menu .menu.parse.m

menubutton .menu.generate -text "Generate" -underline 0 -menu .menu.generate.m
menu .menu.generate.m

if $parser_exists then {
    .menu.parse.m add command -label "Parse sentence" -command {
         if [SP_send_sent] {prolog parse_sent}
      }  -underline 0
}

if $generator_exists then {
    .menu.generate.m add command -label "Generate lf" -command {
      #  global lfs
      #  global lfs
        set lf [SP_qbox .l lfs "Select" "     Select a logical form:     "]
        update
        if {$lf == 0} {return 0}
        prolog "generate_atom $lf"
  } -underline 0
    .menu.generate.m add command -label "Generate object" -command {
        set m [SP_select_obj]
        if {$m == 0} {return 0}
        prolog "generate_object $m"
      } -underline 9
    .menu.generate.m add cascade -label "Generator Selection" \
             -menu .menu.generate.m.generators 
    .menu.generate.m add command -label "Compare Generators on lf" \
        -command {
        set lf [SP_qbox .l lfs \
                 "Select" "          Select a logical form:          "]
        update
        if {$lf == 0} {return 0}
        prolog "generate_atom_comp $lf"
    } -underline 8
    .menu.generate.m add command -label "Compare Generators on object" \
        -command {
	    set m [SP_select_obj]
            if {$m == 0} {return 0}
            prolog "generate_object_comp $m"
                 } -underline 17
    }

pack .menu.parse .menu.generate -side left

menubutton .menu.test -text "Test-Suite" -underline 0 -menu .menu.test.m
menu .menu.test.m -postcommand {
    catch {.menu.test.m delete 0 last}
    prolog test_suite_exists
    if $test_suite_exists then {
        .menu.test.m add command -label "Run test suite" \
                -command {prolog hdrug_go} -underline 0
        .menu.test.m add command -label "Run test suite and View" -underline 4\
	        -command {prolog go_and_table_tk_add}
        .menu.test.m add command -label "Reload test suite" -underline 2\
           -command {
            prolog {compile suite} 
            prolog update_sents
        } 
    }
    if !$test_suite_exists then {
            .menu.test.m add command -label "Load test suite" -underline 0\
           -command {
            prolog {compile suite} 
            prolog update_sents
            }
    }
    prolog test_result_exists
    if $test_result_exists then {

	 .menu.test.m add cascade -label "View test results"\
                -menu .menu.test.view
        .menu.test.m add command -label "Destroy test results"\
            -command {prolog reset_table} -underline 0
    }
    if $parser_exists then {
        .menu.test.m add cascade -label "Parser Selection" \
             -menu .menu.test.m.parsers 
         .menu.test.m add command -label "Compare Parsers" -command {
        if [SP_send_sent] {prolog parse_sent_comp}
      }  -underline 8
    }
}

menu .menu.test.view
.menu.test.view add command -label "Individual (Tk)"\
    -command {prolog sts}
.menu.test.view add command -label "Totals per #words (Tk)"\
    -command {prolog sts_add}
.menu.test.view add command -label "Totals per #words (LaTeX)"\
    -command {prolog p_tt}
.menu.test.view add command -label "Totals per #readings (LaTeX)"\
    -command {prolog p_tt_amb}
.menu.test.view add command -label "Individual (Prolog)"\
    -command {prolog print_table}
.menu.test.view add command -label "Totals per #words (Prolog)"\
    -command {prolog print_table_add}

menu .menu.test.m.parsers -postcommand {
    prolog send_parsers
    catch {.menu.test.m.parsers delete 0 last}
    foreach i $parsers {
        .menu.test.m.parsers add check -label $i -variable parser($i)\
           -command "SP_change_parser $i"
    }
}

menu .menu.generate.m.generators -postcommand {
    prolog send_generators
    catch {.menu.generate.m.generators delete 0 last}
    foreach i $generators {
        .menu.generate.m.generators add check -label $i -variable generator($i)\
           -command "SP_change_generator $i"
    }
}

pack .menu.test -side left

menubutton .menu.grammar -text "Grammar" -underline 0 -menu .menu.grammar.m
menu .menu.grammar.m
.menu.grammar.m add command -label "Compile grammar" \
  -command {
     prolog compile_grammar
     prolog update_types
     prolog update_preds
           } -underline 0
.menu.grammar.m add command -label "Reconsult grammar" \
  -command {
     prolog reconsult_grammar
     prolog update_types
     prolog update_preds
	} -underline 0
.menu.grammar.m add command -label "Compile grammar file"\
  -command {
       global fsBox
       FSBox {} {} {
           prolog "compile_grammar_file $fsBox(path)/$fsBox(name)" 
	   prolog update_types
           prolog update_preds
                   }
       }
.menu.grammar.m add command -label "Reconsult grammar file" \
  -command {
       global fsBox
       FSBox {} {} {
           prolog "reconsult_grammar_file $fsBox(path)/$fsBox(name)" 
           prolog update_types
           prolog update_preds
                   }
       }

pack .menu.grammar -side left

# Viewing Objects, Predicates and Type definitions
menubutton .menu.show -text "View" -menu .menu.show.m -underline 0
menu .menu.show.m -postcommand {
    catch {.menu.show.m delete 0 last}
    prolog object_exists
    prolog predicate_exists
    prolog type_exists
    if $object_exists then {
        .menu.show.m add cascade -label Object -menu .menu.show.m.object
    }
    if $predicate_exists then {
        .menu.show.m add cascade -label Predicate \
           -menu .menu.show.m.predicate
    }
    if $type_exists then {
        .menu.show.m add cascade -label Type -menu .menu.show.m.type
    }
}


# Viewing objects
menu .menu.show.m.object
.menu.show.m.object add cascade -label "Tk" \
      -menu .menu.show.m.object.canvas
.menu.show.m.object add cascade -label "Prolog" \
      -menu .menu.show.m.object.prolog
.menu.show.m.object add cascade -label "LaTeX" \
      -menu .menu.show.m.object.latex

menu .menu.show.m.object.canvas -postcommand {
    prolog send_treedefs
    catch {.menu.show.m.object.canvas delete 0 last}
    foreach i $treedefs {
            .menu.show.m.object.canvas add command -label Tree($i) \
              -command "show_object_canvas_tree $i"
    }
}

menu .menu.show.m.object.prolog -postcommand {
    prolog send_treedefs
    prolog type_exists
    catch {.menu.show.m.object.prolog delete 0 last}
    foreach i $treedefs {
            .menu.show.m.object.prolog add command -label Tree($i) \
              -command "show_object_prolog_tree $i"
    }
    .menu.show.m.object.prolog add command -label Semantics \
              -command {
       set obj [SP_select_obj]
       if {$obj == 0} {return 0}
       prolog "show_object_prolog_semantics $obj"
    }

    if $type_exists then {
        .menu.show.m.object.prolog add command -label Matrix \
                  -command {
           set obj [SP_select_obj]
           if {$obj == 0} {return 0}
           prolog "show_object_prolog_matrix $obj"
           }
    }
    .menu.show.m.object.prolog add command -label Text \
              -command {
         set obj [SP_select_obj]
         if {$obj == 0} {return 0}
         prolog "show_object_prolog_text $obj"
         }
}

menu .menu.show.m.object.latex -postcommand {
    prolog send_treedefs
    catch {.menu.show.m.object.latex delete 0 last}
    foreach i $treedefs {
            .menu.show.m.object.latex add command -label Tree($i) \
              -command "show_object_latex_tree $i"
    }
    if $type_exists then {
        .menu.show.m.object.latex add command -label Matrix \
                  -command {
          set obj [SP_select_obj]
          if {$obj == 0} {return 0}
          prolog "show_object_latex_matrix $obj"
          }
    }
    .menu.show.m.object.latex add command -label Semantics \
              -command {
         set obj [SP_select_obj]
         if {$obj == 0} {return 0}
         prolog "show_object_latex_semantics $obj"
         }
    .menu.show.m.object.latex add command -label Text \
              -command {
        set obj [SP_select_obj]
        if {$obj == 0} {return 0}
        prolog "show_object_latex_text $obj"
        }
}

# Viewing Predicates
menu .menu.show.m.predicate

.menu.show.m.predicate add cascade -label "Prolog" \
      -menu .menu.show.m.predicate.prolog
.menu.show.m.predicate add cascade -label "LaTeX" \
      -menu .menu.show.m.predicate.latex

menu .menu.show.m.predicate.latex
.menu.show.m.predicate.latex add command -label "Text" \
      -command {
    set pred [SP_qlist .w $preds Pred "Pred to show:" ]
    if {$pred == 0} {return 0}
    prolog "show_predicate_latex_text $pred"
}

if $type_exists then {
    .menu.show.m.predicate.latex add command -label "Matrix" \
          -command {
    set pred [SP_qlist .w $preds Pred "Pred to show:" ]
    if {$pred == 0} {return 0}
    prolog "show_predicate_latex_matrix $pred"
    }
}

menu .menu.show.m.predicate.prolog
.menu.show.m.predicate.prolog add command -label "Text" \
      -command {
    set pred [SP_qlist .w $preds Pred "Pred to show" ]
    if {$pred == 0} {return 0}
    prolog "show_predicate_prolog_text $pred"
}
if $type_exists then {
    .menu.show.m.predicate.prolog add command -label "Matrix" \
          -command {
    set pred [SP_qlist .w $preds Pred "Pred to show:" ]
    if {$pred == 0} {return 0}
    prolog "show_predicate_prolog_matrix $pred"
     }
}

# Viewing type definitions
menu .menu.show.m.type
.menu.show.m.type add command -label "Tk" -command {
    set type [SP_qlist .w $types Type "Type to show" ]
    if {$type == 0} {return 0}
    prolog "show_type_canvas $type"
}

.menu.show.m.type add cascade -label "Prolog" -menu .menu.show.m.type.prolog
.menu.show.m.type add command -label "LaTeX"     -command {
    set type [SP_qlist .w $types Type "Type to show" ]
    if {$type == 0} {return 0}
    prolog "show_type_latex $type"
}


menu .menu.show.m.type.prolog
.menu.show.m.type.prolog add command -label "Tree" -command {
    set type [SP_qlist .w $types Type "Type to show"]
    if {$type == 0} {return 0}
    prolog "show_type_prolog_tree $type"
}

.menu.show.m.type.prolog add command -label "Text" -command {
    set type [SP_qlist .w $types Type "Type to show"]
    if {$type == 0} {return 0}
    prolog "show_type_prolog_text $type"
}

pack .menu.show -side left

bind . <Any-FocusIn> {
    if {("%d" == "NotifyVirtual") && ("%m" == "NotifyNormal")} {
	focus .menu
    }
}

tk_menuBar .menu .menu.help .menu.file .menu.debug .menu.options \
                 .menu.parse .menu.generate .menu.test .menu.grammar \
                 .menu.show

# Alt or Control does not work!!!
bind .menu <Key> {tk_traverseToMenu %W %A}


#################
### ButtonBar ###
#################

menubutton .t.topt -menu .t.topt.menu -text Top -relief raised
label .t.top -textvariable flag(top_features) -width 10
pack .t.topt .t.top -side left


if $parser_exists then {
    menubutton .t.parsert -menu .t.parsert.menu -text Parser -relief raised
    label .t.parser -textvariable flag(parser) -width 10
    pack .t.parsert .t.parser -side left
    menu .t.parsert.menu -postcommand {
      prolog {send_list parser_mode parsers}
      catch {.t.parsert.menu delete 0 last}
      foreach i $parsers {
          .t.parsert.menu add command -label $i \
                 -command "SP_receive_flag parser $i"
      }
    }
}

if $generator_exists then {
    menubutton .t.generatort -menu .t.generatort.menu -text Generator -relief raised
    label .t.generator -textvariable flag(generator) -width 10
    pack .t.generatort .t.generator -side left
    menu .t.generatort.menu -postcommand {
      prolog {send_list generator_mode generators}
      catch {.t.generatort.menu delete 0 last}
      foreach i $generators {
          .t.generatort.menu add command -label $i \
                 -command "SP_receive_flag generator $i"
      }
   }
}

menu .t.topt.menu -postcommand {
    prolog send_topcats
    catch {.t.topt.menu delete 0 last}
    foreach i $topcats {
        .t.topt.menu add command -label $i \
               -command "SP_receive_flag top_features $i"
    }
    .t.topt.menu add command -label "<No Top>" \
               -command "SP_receive_flag top_features undefined"

}

#################
### ObjectBar ###
#################

# this one here, to be able to overwrite it in an application..
proc SP_object {no} {
    # should not be necc:
    catch "destroy .bb.obj$no"

    global treedefs
    global type_exists
    global generator_exists
    prolog send_treedefs
    prolog type_exists

    menubutton .bb.obj$no -menu .bb.obj$no.a -text "$no" -bg yellow -relief raised
    pack .bb.obj$no -side left
    menu .bb.obj$no.a
    .bb.obj$no.a add cascade -label "View" \
	            -menu .bb.obj$no.a.show
    menu .bb.obj$no.a.show
    .bb.obj$no.a.show add cascade -label "Tk" \
                    -menu .bb.obj$no.a.show.canvas
      menu .bb.obj$no.a.show.canvas
      catch {.bb.obj$no.a.show.canvas delete 0 last}
      foreach i $treedefs {
              .bb.obj$no.a.show.canvas add command -label Tree($i) \
                -command "SP_show_object_canvas_tree $i $no"
      }
      .bb.obj$no.a.show add cascade -label "Prolog" \
        -menu .bb.obj$no.a.show.prolog
      menu .bb.obj$no.a.show.prolog
      catch {.bb.obj$no.a.show.prolog delete 0 last}
      foreach i $treedefs {
              .bb.obj$no.a.show.prolog add command -label Tree($i) \
                -command "SP_show_object_prolog_tree $i $no"
      }
      .bb.obj$no.a.show.prolog add command -label Semantics \
                -command "SP_show_object_prolog_semantics $no"
      if $type_exists then {
          .bb.obj$no.a.show.prolog add command -label Matrix \
                    -command "SP_show_object_prolog_matrix $no"
      }
      .bb.obj$no.a.show.prolog add command -label Text \
                -command "SP_show_object_prolog_text $no"
      
      .bb.obj$no.a.show add cascade -label "LaTeX" \
                             -menu .bb.obj$no.a.show.latex
      catch {.bb.obj$no.a.show.latex delete 0 last}
      menu .bb.obj$no.a.show.latex
      foreach i $treedefs {
              .bb.obj$no.a.show.latex add command -label Tree($i) \
                         -command "SP_show_object_latex_tree $i $no"
      }
      if $type_exists then {
          .bb.obj$no.a.show.latex add command -label Matrix \
                    -command "SP_show_object_latex_matrix $no"
      }
      .bb.obj$no.a.show.latex add command -label Text \
                -command "SP_show_object_latex_text $no"
      .bb.obj$no.a.show.latex add command -label Semantics \
                -command "SP_show_object_latex_semantics $no"
      if $generator_exists then {
          .bb.obj$no.a add command -label Generate \
                    -command "SP_generate_object $no"
          .bb.obj$no.a add command -label "Compare Generators" \
                    -command "SP_generate_object_comp $no"
           }
    update
}

# put objects in ObjectBar if they already exist
set i 1
while {$i < $objs} {
        SP_object $i
        incr i +1
}
unset i

#####################################
# obtain some more info from prolog #
#####################################

prolog hdrug_startup_hook_end

wm deiconify .
update


